# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core.commands.client_factory import get_mgmt_service_client
from azure.cli.core.profiles import ResourceType
from azure.cli.core.auth.identity import get_environment_credential, AZURE_CLIENT_ID

# pylint: disable=import-outside-toplevel

RM_URI_OVERRIDE = 'AZURE_CLI_POSTGRESQL_FLEXIBLE_RM_URI'
SUB_ID_OVERRIDE = 'AZURE_CLI_POSTGRESQL_FLEXIBLE_SUB_ID'


def get_postgresql_flexible_management_client(cli_ctx, subscription_id=None, **_):
    from os import getenv
    from azure.mgmt.postgresqlflexibleservers import PostgreSQLManagementClient
    # Allow overriding resource manager URI using environment variable
    # for testing purposes. Subscription id is also determined by environment
    # variable.
    rm_uri_override = getenv(RM_URI_OVERRIDE)
    subscription = subscription_id if subscription_id is not None else getenv(SUB_ID_OVERRIDE)
    if rm_uri_override:
        client_id = getenv(AZURE_CLIENT_ID)
        if client_id:
            credentials = get_environment_credential()
        else:
            from msrest.authentication import Authentication  # pylint: disable=import-error
            credentials = Authentication()

        return PostgreSQLManagementClient(
            subscription_id=subscription,
            base_url=rm_uri_override,
            credential=credentials)
    # Normal production scenario.
    return get_mgmt_service_client(cli_ctx, PostgreSQLManagementClient, subscription_id=subscription)


def cf_postgres_flexible_servers(cli_ctx, _):
    return get_postgresql_flexible_management_client(cli_ctx).servers


def cf_postgres_flexible_firewall_rules(cli_ctx, _):
    return get_postgresql_flexible_management_client(cli_ctx).firewall_rules


def cf_postgres_flexible_virtual_endpoints(cli_ctx, _):
    return get_postgresql_flexible_management_client(cli_ctx).virtual_endpoints


def cf_postgres_flexible_config(cli_ctx, _):
    return get_postgresql_flexible_management_client(cli_ctx).configurations


def cf_postgres_flexible_replica(cli_ctx, _):
    return get_postgresql_flexible_management_client(cli_ctx).replicas


def cf_postgres_flexible_location_capabilities(cli_ctx, _):
    return get_postgresql_flexible_management_client(cli_ctx).capabilities_by_location


def cf_postgres_flexible_server_capabilities(cli_ctx, _):
    return get_postgresql_flexible_management_client(cli_ctx).capabilities_by_server


def cf_postgres_flexible_backups(cli_ctx, _):
    return get_postgresql_flexible_management_client(cli_ctx).backups_automatic_and_on_demand


def cf_postgres_flexible_operations(cli_ctx, _):
    return get_postgresql_flexible_management_client(cli_ctx).operations


def cf_postgres_flexible_admin(cli_ctx, _):
    return get_postgresql_flexible_management_client(cli_ctx).administrators_microsoft_entra


def cf_postgres_flexible_migrations(cli_ctx, _):
    return get_postgresql_flexible_management_client(cli_ctx).migrations


def cf_postgres_flexible_server_threat_protection_settings(cli_ctx, _):
    return get_postgresql_flexible_management_client(cli_ctx).server_threat_protection_settings


def cf_postgres_flexible_advanced_threat_protection_settings(cli_ctx, _):
    return get_postgresql_flexible_management_client(cli_ctx).advanced_threat_protection_settings


def cf_postgres_flexible_server_log_files(cli_ctx, _):
    return get_postgresql_flexible_management_client(cli_ctx).captured_logs


def cf_postgres_check_resource_availability(cli_ctx, _):
    return get_postgresql_flexible_management_client(cli_ctx).name_availability


def cf_postgres_flexible_db(cli_ctx, _):
    return get_postgresql_flexible_management_client(cli_ctx).databases


def cf_postgres_flexible_private_dns_zone_suffix_operations(cli_ctx, _):
    return get_postgresql_flexible_management_client(cli_ctx).private_dns_zone_suffix


def cf_postgres_flexible_private_endpoint_connections(cli_ctx, _):
    return get_postgresql_flexible_management_client(cli_ctx).private_endpoint_connections


def cf_postgres_flexible_private_link_resources(cli_ctx, _):
    return get_postgresql_flexible_management_client(cli_ctx).private_link_resources


def cf_postgres_flexible_tuning_options(cli_ctx, _):
    return get_postgresql_flexible_management_client(cli_ctx).tuning_options


def cf_postgres_flexible_maintenance_events(cli_ctx, _):
    return get_postgresql_flexible_management_client(cli_ctx).maintenance_events


def resource_client_factory(cli_ctx, subscription_id=None):
    return get_mgmt_service_client(cli_ctx, ResourceType.MGMT_RESOURCE_RESOURCES, subscription_id=subscription_id)


def private_dns_client_factory(cli_ctx, subscription_id=None):
    from azure.mgmt.privatedns import PrivateDnsManagementClient
    return get_mgmt_service_client(cli_ctx, PrivateDnsManagementClient, subscription_id=subscription_id).private_zones


def private_dns_link_client_factory(cli_ctx, subscription_id=None):
    from azure.mgmt.privatedns import PrivateDnsManagementClient
    return get_mgmt_service_client(cli_ctx, PrivateDnsManagementClient,
                                   subscription_id=subscription_id).virtual_network_links



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import json
from azure.cli.testsdk import ScenarioTest, JMESPathCheck, ResourceGroupPreparer, StorageAccountPreparer
from azure.core.exceptions import HttpResponseError
from azure.cli.core.azclierror import ArgumentUsageError

class TestMonitorDiagnosticSettings(ScenarioTest):
    @ResourceGroupPreparer(location='southcentralus')
    @StorageAccountPreparer(location='southcentralus')
    def test_monitor_diagnostic_settings_scenario(self, resource_group, storage_account):

        self.kwargs.update({
            'nsg': self.create_random_name(prefix='nsg', length=16),
            'ws': self.create_random_name(prefix='ws', length=16)
        })
        self.cmd('network nsg create -n {nsg} -g {rg}')
        self.cmd('monitor log-analytics workspace create -g {rg} -n {ws}')

        self.cmd('monitor diagnostic-settings categories list -g {rg} --resource-type Microsoft.Network/networkSecurityGroups --resource {nsg}', checks=[
            self.check('length(value)', 2)
        ])

        # test diagnostic-settings categories show
        self.cmd(
            'monitor diagnostic-settings categories show -n NetworkSecurityGroupEvent -g {rg} --resource-type Microsoft.Network/networkSecurityGroups --resource {nsg}',
            checks=[
                self.check('categoryType', 'Logs')
            ])

        self.kwargs['log_config'] = json.dumps([
            {
                "category": "NetworkSecurityGroupEvent",
                "enabled": True,
                "retentionPolicy": {"days": 0, "enabled": False}
            },
            {
                "category": "NetworkSecurityGroupRuleCounter",
                "enabled": True,
                "retentionPolicy": {"days": 0, "enabled": False}
            }
        ])

        self.cmd('monitor diagnostic-settings create -n test01 --resource {nsg} --resource-type Microsoft.Network/networkSecurityGroups --resource-group {rg} --storage-account {sa} --logs \'{log_config}\' -o json',
                 checks=self.check('name', 'test01'))

        self.cmd('monitor diagnostic-settings list --resource {nsg} --resource-type Microsoft.Network/networkSecurityGroups --resource-group {rg} -o json', checks=[
            self.check('length(@)', 1),
            self.check('[0].name', 'test01')
        ])

        self.cmd('monitor diagnostic-settings show -n test01 --resource {nsg} --resource-type Microsoft.Network/networkSecurityGroups --resource-group {rg} -o json',
                 checks=self.check('name', 'test01'))

        self.cmd('monitor diagnostic-settings delete -n test01 --resource {nsg} --resource-type Microsoft.Network/networkSecurityGroups --resource-group {rg} -o json')

        self.cmd('monitor diagnostic-settings list --resource {nsg} --resource-type Microsoft.Network/networkSecurityGroups --resource-group {rg} -o json',
                 checks=self.check('length(@)', 0))

        self.cmd('monitor diagnostic-settings create -n test02 --resource {nsg} --resource-type Microsoft.Network/networkSecurityGroups --resource-group {rg} --workspace {ws} --export-to-resource-specific --logs \'{log_config}\' -o json',
                 checks=[
                     self.check('name', 'test02'),
                     self.check('logAnalyticsDestinationType', 'Dedicated')
                 ])

    @ResourceGroupPreparer(location='southcentralus')
    def test_monitor_diagnostic_settings_marketplace_id(self, resource_group):
        self.kwargs.update({
            'nsg': self.create_random_name(prefix='nsg', length=16),
            'ws': self.create_random_name(prefix='ws', length=16),
            'sa': self.create_random_name(prefix='sa', length=16),
            'sa1': self.create_random_name(prefix='sa', length=16),
            'vm1': 'vm1',
            'vmss': 'vmss1'
        })
        self.cmd('network nsg create -n {nsg} -g {rg}')

        #self.kwargs['mp_id'] = self.cmd('network nsg show -n {nsg} -g {rg}').get_output_in_json()['id']
        #self.cmd('vmss create -g {rg} -n {vmss} --image Canonical:UbuntuServer:18.04-LTS:latest --admin-username testadmin --admin-password TestTest12#$')
        #self.kwargs['mp_id'] = self.cmd('vmss show -g {rg} -n {vmss}').get_output_in_json()['id']
        # self.kwargs['mp_id'] = self.cmd('vm create -g {rg} -n {vm1} --image Canonical:UbuntuServer:18.04-LTS:latest --admin-password TestPassword11!! '
        #                     '--admin-username testadmin --authentication-type password').get_output_in_json()['id']

        # self.cmd('monitor log-analytics workspace create -g {rg} -n {ws}')
        # self.kwargs['mp_id'] = self.cmd('monitor log-analytics workspace show -g {rg} -n {ws}').get_output_in_json()['id']
        self.kwargs['mp_id'] = self.cmd('az storage account create -n {sa1} -g {rg} -l westus --sku Standard_LRS --kind Storage --https-only').get_output_in_json()['id']
        self.cmd('az storage account create -n {sa} -g {rg} -l westus --sku Standard_LRS --kind Storage --https-only')
        # self.kwargs['mp_id'] = "/subscriptions/0b1f6471-1bf0-4dda-aec3-cb9272f09590/resourceGroups/test-rg/providers/Microsoft.Datadog/monitors/dd1"

        self.kwargs['log_config'] = json.dumps([
            {
                "category": "NetworkSecurityGroupEvent",
                "enabled": True,
                "retention-policy": {"days": 0, "enabled": False}
            },
            {
                "category": "NetworkSecurityGroupRuleCounter",
                "enabled": True,
                "retention-policy": {"days": 0, "enabled": False}
            }
        ])
        with self.assertRaisesRegex(HttpResponseError, "The resource type 'microsoft.storage/storageaccounts' is not a marketplace resource."):
            self.cmd('monitor diagnostic-settings create -n test01 --resource {nsg} --resource-type Microsoft.Network/networkSecurityGroups --resource-group {rg} --storage-account {sa} --marketplace-partner-id {mp_id} --logs \'{log_config}\' -o json')

    @ResourceGroupPreparer(location='southcentralus')
    @StorageAccountPreparer(location='southcentralus')
    def test_monitor_diagnostic_settings_resource_id(self, resource_group, storage_account):
        self.kwargs.update({
            'nsg': self.create_random_name(prefix='nsg', length=16)
        })
        self.cmd('network nsg create -n {nsg} -g {rg}')
        self.kwargs['resource_id'] = self.cmd('network nsg show -n {nsg} -g {rg}').get_output_in_json()['id']

        self.cmd('monitor diagnostic-settings create -n test01 --resource {resource_id} --storage-account {sa} -o json',
                 checks=self.check('name', 'test01'))

    @ResourceGroupPreparer(location='southcentralus')
    @StorageAccountPreparer(location='southcentralus')
    def test_monitor_diagnostic_settings_workspace(self, resource_group, storage_account):

        self.kwargs.update({
            'nsg': self.create_random_name(prefix='nsg', length=16),
            'ws': self.create_random_name(prefix='ws', length=16)
        })
        self.cmd('network nsg create -n {nsg} -g {rg}')
        self.kwargs['resource_id'] = self.cmd('network nsg show -n {nsg} -g {rg}').get_output_in_json()['id']
        self.cmd('monitor log-analytics workspace create -g {rg} -n {ws}')

        self.cmd('monitor diagnostic-settings create -n test01 --resource {resource_id} --storage-account {sa} --workspace {ws} --export-to-resource-specific -o json',
                 checks=[
                     self.check('name', 'test01'),
                     self.check('logAnalyticsDestinationType', 'Dedicated')
                 ])

        with self.assertRaises(ArgumentUsageError):
            self.cmd('monitor diagnostic-settings create -n test02 --resource {resource_id} --storage-account {sa} --export-to-resource-specific -o json')


if __name__ == '__main__':
    import unittest

    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "network traffic-manager profile create",
)
class Create(AAZCommand):
    """Create a Traffic Manager profile.

    :example: Create a traffic manager profile with performance routing.
        az network traffic-manager profile create -g MyResourceGroup -n MyTmProfile --routing-method Performance --unique-dns-name mywebapp --ttl 30 --protocol HTTP --port 80 --path "/"

    :example: Create a traffic manager profile.
        az network traffic-manager profile create -n MyTmProfile -g MyResourceGroup --routing-method subnet --unique-dns-name mywebapp --custom-headers [{name:foo,value:bar}] --status-code-ranges [{min:200,max:202}] --path "/"
    """

    _aaz_info = {
        "version": "2024-04-01-preview",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.network/trafficmanagerprofiles/{}", "2024-04-01-preview"],
        ]
    }

    def _handler(self, command_args):
        super()._handler(command_args)
        self._execute_operations()
        return self._output()

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.name = AAZStrArg(
            options=["-n", "--name"],
            help="Traffic manager profile name.",
            required=True,
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )

        # define Arg Group "DnsConfig"

        _args_schema = cls._args_schema
        _args_schema.unique_dns_name = AAZStrArg(
            options=["--unique-dns-name"],
            arg_group="DnsConfig",
            help="Relative DNS name for the traffic manager profile. Resulting FQDN will be `<unique-dns-name>.trafficmanager.net` and must be globally unique.",
        )
        _args_schema.ttl = AAZIntArg(
            options=["--ttl"],
            arg_group="DnsConfig",
            help="DNS config time-to-live in seconds.",
            default=30,
        )

        # define Arg Group "Monitor Configuration"

        _args_schema = cls._args_schema
        _args_schema.custom_headers = AAZListArg(
            options=["--custom-headers"],
            arg_group="Monitor Configuration",
            help="Space-separated list of names or values.",
        )
        _args_schema.status_code_ranges = AAZListArg(
            options=["--status-code-ranges"],
            arg_group="Monitor Configuration",
            help="Space-separated list of status codes in shorthand-syntax format.",
        )
        _args_schema.interval = AAZIntArg(
            options=["--interval"],
            arg_group="Monitor Configuration",
            help="The interval in seconds at which health checks are conducted.",
        )
        _args_schema.path = AAZStrArg(
            options=["--path"],
            arg_group="Monitor Configuration",
            help="Path to monitor. Use \"\"('\"\"' in PowerShell) for none.",
        )
        _args_schema.port = AAZIntArg(
            options=["--port"],
            arg_group="Monitor Configuration",
            help="Port to monitor.",
            default=80,
        )
        _args_schema.protocol = AAZStrArg(
            options=["--protocol"],
            arg_group="Monitor Configuration",
            help="Monitor protocol.",
            default="HTTP",
            enum={"HTTP": "HTTP", "HTTPS": "HTTPS", "TCP": "TCP"},
        )
        _args_schema.timeout = AAZIntArg(
            options=["--timeout"],
            arg_group="Monitor Configuration",
            help="The time in seconds allowed for endpoints to respond to a health check.",
        )
        _args_schema.max_failures = AAZIntArg(
            options=["--max-failures"],
            arg_group="Monitor Configuration",
            help="The number of consecutive failed health checks tolerated before an endpoint is considered degraded.",
        )

        custom_headers = cls._args_schema.custom_headers
        custom_headers.Element = AAZObjectArg()

        _element = cls._args_schema.custom_headers.Element
        _element.name = AAZStrArg(
            options=["name"],
            help="Header name.",
        )
        _element.value = AAZStrArg(
            options=["value"],
            help="Header value.",
        )

        status_code_ranges = cls._args_schema.status_code_ranges
        status_code_ranges.Element = AAZObjectArg()

        _element = cls._args_schema.status_code_ranges.Element
        _element.max = AAZIntArg(
            options=["max"],
            help="Max status code.",
        )
        _element.min = AAZIntArg(
            options=["min"],
            help="Min status code.",
        )

        # define Arg Group "MonitorConfig"

        # define Arg Group "Parameters"

        _args_schema = cls._args_schema
        _args_schema.location = AAZResourceLocationArg(
            arg_group="Parameters",
            help="The Azure Region where the resource lives",
            default="global",
            fmt=AAZResourceLocationArgFormat(
                resource_group_arg="resource_group",
            ),
        )
        _args_schema.tags = AAZDictArg(
            options=["--tags"],
            arg_group="Parameters",
            help="Space-separated tags: key[=value] [key[=value] ...].",
        )

        tags = cls._args_schema.tags
        tags.Element = AAZStrArg()

        # define Arg Group "Properties"

        _args_schema = cls._args_schema
        _args_schema.max_return = AAZIntArg(
            options=["--max-return"],
            arg_group="Properties",
            help="Maximum number of endpoints to be returned for MultiValue routing type.",
        )
        _args_schema.status = AAZStrArg(
            options=["--status"],
            arg_group="Properties",
            help="Status of the Traffic Manager profile.",
            default="Enabled",
            enum={"Disabled": "Disabled", "Enabled": "Enabled"},
        )
        _args_schema.record_type = AAZStrArg(
            options=["--record-type"],
            arg_group="Properties",
            help="When record type is set, a traffic manager profile will allow only endpoints that match this type.",
            enum={"A": "A", "AAAA": "AAAA", "CNAME": "CNAME"},
        )
        _args_schema.routing_method = AAZStrArg(
            options=["--routing-method"],
            arg_group="Properties",
            help="Routing method.",
            enum={"Geographic": "Geographic", "MultiValue": "MultiValue", "Performance": "Performance", "Priority": "Priority", "Subnet": "Subnet", "Weighted": "Weighted"},
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        self.ProfilesCreateOrUpdate(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class ProfilesCreateOrUpdate(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200, 201]:
                return self.on_200_201(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/trafficmanagerprofiles/{profileName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "PUT"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "profileName", self.ctx.args.name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2024-04-01-preview",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Content-Type", "application/json",
                ),
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        @property
        def content(self):
            _content_value, _builder = self.new_content_builder(
                self.ctx.args,
                typ=AAZObjectType,
                typ_kwargs={"flags": {"required": True, "client_flatten": True}}
            )
            _builder.set_prop("location", AAZStrType, ".location")
            _builder.set_prop("name", AAZStrType, ".name")
            _builder.set_prop("properties", AAZObjectType, typ_kwargs={"flags": {"client_flatten": True}})
            _builder.set_prop("tags", AAZDictType, ".tags")

            properties = _builder.get(".properties")
            if properties is not None:
                properties.set_prop("dnsConfig", AAZObjectType)
                properties.set_prop("maxReturn", AAZIntType, ".max_return")
                properties.set_prop("monitorConfig", AAZObjectType)
                properties.set_prop("profileStatus", AAZStrType, ".status")
                properties.set_prop("recordType", AAZStrType, ".record_type")
                properties.set_prop("trafficRoutingMethod", AAZStrType, ".routing_method")

            dns_config = _builder.get(".properties.dnsConfig")
            if dns_config is not None:
                dns_config.set_prop("relativeName", AAZStrType, ".unique_dns_name")
                dns_config.set_prop("ttl", AAZIntType, ".ttl")

            monitor_config = _builder.get(".properties.monitorConfig")
            if monitor_config is not None:
                monitor_config.set_prop("customHeaders", AAZListType, ".custom_headers")
                monitor_config.set_prop("expectedStatusCodeRanges", AAZListType, ".status_code_ranges")
                monitor_config.set_prop("intervalInSeconds", AAZIntType, ".interval")
                monitor_config.set_prop("path", AAZStrType, ".path")
                monitor_config.set_prop("port", AAZIntType, ".port")
                monitor_config.set_prop("protocol", AAZStrType, ".protocol")
                monitor_config.set_prop("timeoutInSeconds", AAZIntType, ".timeout")
                monitor_config.set_prop("toleratedNumberOfFailures", AAZIntType, ".max_failures")

            custom_headers = _builder.get(".properties.monitorConfig.customHeaders")
            if custom_headers is not None:
                custom_headers.set_elements(AAZObjectType, ".")

            _elements = _builder.get(".properties.monitorConfig.customHeaders[]")
            if _elements is not None:
                _elements.set_prop("name", AAZStrType, ".name")
                _elements.set_prop("value", AAZStrType, ".value")

            expected_status_code_ranges = _builder.get(".properties.monitorConfig.expectedStatusCodeRanges")
            if expected_status_code_ranges is not None:
                expected_status_code_ranges.set_elements(AAZObjectType, ".")

            _elements = _builder.get(".properties.monitorConfig.expectedStatusCodeRanges[]")
            if _elements is not None:
                _elements.set_prop("max", AAZIntType, ".max")
                _elements.set_prop("min", AAZIntType, ".min")

            tags = _builder.get(".tags")
            if tags is not None:
                tags.set_elements(AAZStrType, ".")

            return self.serialize_content(_content_value)

        def on_200_201(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200_201
            )

        _schema_on_200_201 = None

        @classmethod
        def _build_schema_on_200_201(cls):
            if cls._schema_on_200_201 is not None:
                return cls._schema_on_200_201

            cls._schema_on_200_201 = AAZObjectType()

            _schema_on_200_201 = cls._schema_on_200_201
            _schema_on_200_201.id = AAZStrType()
            _schema_on_200_201.location = AAZStrType()
            _schema_on_200_201.name = AAZStrType()
            _schema_on_200_201.properties = AAZObjectType(
                flags={"client_flatten": True},
            )
            _schema_on_200_201.tags = AAZDictType()
            _schema_on_200_201.type = AAZStrType()

            properties = cls._schema_on_200_201.properties
            properties.allowed_endpoint_record_types = AAZListType(
                serialized_name="allowedEndpointRecordTypes",
            )
            properties.dns_config = AAZObjectType(
                serialized_name="dnsConfig",
            )
            properties.endpoints = AAZListType()
            properties.max_return = AAZIntType(
                serialized_name="maxReturn",
            )
            properties.monitor_config = AAZObjectType(
                serialized_name="monitorConfig",
            )
            properties.profile_status = AAZStrType(
                serialized_name="profileStatus",
            )
            properties.record_type = AAZStrType(
                serialized_name="recordType",
            )
            properties.traffic_routing_method = AAZStrType(
                serialized_name="trafficRoutingMethod",
            )
            properties.traffic_view_enrollment_status = AAZStrType(
                serialized_name="trafficViewEnrollmentStatus",
            )

            allowed_endpoint_record_types = cls._schema_on_200_201.properties.allowed_endpoint_record_types
            allowed_endpoint_record_types.Element = AAZStrType()

            dns_config = cls._schema_on_200_201.properties.dns_config
            dns_config.fqdn = AAZStrType(
                flags={"read_only": True},
            )
            dns_config.relative_name = AAZStrType(
                serialized_name="relativeName",
            )
            dns_config.ttl = AAZIntType()

            endpoints = cls._schema_on_200_201.properties.endpoints
            endpoints.Element = AAZObjectType()

            _element = cls._schema_on_200_201.properties.endpoints.Element
            _element.id = AAZStrType()
            _element.name = AAZStrType()
            _element.properties = AAZObjectType(
                flags={"client_flatten": True},
            )
            _element.type = AAZStrType()

            properties = cls._schema_on_200_201.properties.endpoints.Element.properties
            properties.always_serve = AAZStrType(
                serialized_name="alwaysServe",
            )
            properties.custom_headers = AAZListType(
                serialized_name="customHeaders",
            )
            properties.endpoint_location = AAZStrType(
                serialized_name="endpointLocation",
            )
            properties.endpoint_monitor_status = AAZStrType(
                serialized_name="endpointMonitorStatus",
            )
            properties.endpoint_status = AAZStrType(
                serialized_name="endpointStatus",
            )
            properties.geo_mapping = AAZListType(
                serialized_name="geoMapping",
            )
            properties.min_child_endpoints = AAZIntType(
                serialized_name="minChildEndpoints",
            )
            properties.min_child_endpoints_i_pv4 = AAZIntType(
                serialized_name="minChildEndpointsIPv4",
            )
            properties.min_child_endpoints_i_pv6 = AAZIntType(
                serialized_name="minChildEndpointsIPv6",
            )
            properties.priority = AAZIntType()
            properties.subnets = AAZListType()
            properties.target = AAZStrType()
            properties.target_resource_id = AAZStrType(
                serialized_name="targetResourceId",
            )
            properties.weight = AAZIntType()

            custom_headers = cls._schema_on_200_201.properties.endpoints.Element.properties.custom_headers
            custom_headers.Element = AAZObjectType()

            _element = cls._schema_on_200_201.properties.endpoints.Element.properties.custom_headers.Element
            _element.name = AAZStrType()
            _element.value = AAZStrType()

            geo_mapping = cls._schema_on_200_201.properties.endpoints.Element.properties.geo_mapping
            geo_mapping.Element = AAZStrType()

            subnets = cls._schema_on_200_201.properties.endpoints.Element.properties.subnets
            subnets.Element = AAZObjectType()

            _element = cls._schema_on_200_201.properties.endpoints.Element.properties.subnets.Element
            _element.first = AAZStrType()
            _element.last = AAZStrType()
            _element.scope = AAZIntType()

            monitor_config = cls._schema_on_200_201.properties.monitor_config
            monitor_config.custom_headers = AAZListType(
                serialized_name="customHeaders",
            )
            monitor_config.expected_status_code_ranges = AAZListType(
                serialized_name="expectedStatusCodeRanges",
            )
            monitor_config.interval_in_seconds = AAZIntType(
                serialized_name="intervalInSeconds",
            )
            monitor_config.path = AAZStrType()
            monitor_config.port = AAZIntType()
            monitor_config.profile_monitor_status = AAZStrType(
                serialized_name="profileMonitorStatus",
            )
            monitor_config.protocol = AAZStrType()
            monitor_config.timeout_in_seconds = AAZIntType(
                serialized_name="timeoutInSeconds",
            )
            monitor_config.tolerated_number_of_failures = AAZIntType(
                serialized_name="toleratedNumberOfFailures",
            )

            custom_headers = cls._schema_on_200_201.properties.monitor_config.custom_headers
            custom_headers.Element = AAZObjectType()

            _element = cls._schema_on_200_201.properties.monitor_config.custom_headers.Element
            _element.name = AAZStrType()
            _element.value = AAZStrType()

            expected_status_code_ranges = cls._schema_on_200_201.properties.monitor_config.expected_status_code_ranges
            expected_status_code_ranges.Element = AAZObjectType()

            _element = cls._schema_on_200_201.properties.monitor_config.expected_status_code_ranges.Element
            _element.max = AAZIntType()
            _element.min = AAZIntType()

            tags = cls._schema_on_200_201.tags
            tags.Element = AAZStrType()

            return cls._schema_on_200_201


class _CreateHelper:
    """Helper class for Create"""


__all__ = ["Create"]



#!/usr/bin/env python

# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------


import ast
import logging
import re
import sys


# http://noyobo.com/2015/11/13/ANSI-escape-code.html
# Reset all to default: 0
# Bold or increased intensity: 1
# Fraktur (Gothic): 20
# red: 31
# green: 32
# yellow: 33
# grey: 38
reset = "\x1b[0m"
red = "\x1b[31;20m"
bold_red = "\x1b[31;1m"
green = "\x1b[32;20m"
yellow = "\x1b[33;20m"
grey = "\x1b[38;20m"
format = "%(message)s"
TITLE = sys.argv[1]
BODY = sys.argv[2]
words_to_check = {
            'Add': r'\b(added|adding|adds)\b',
            'Allow': r'\b(allowed|allowing|allows)\b',
            'Change': r'\b(changed|changing|changes)\b',
            'Deprecate': r'\b(deprecated|deprecating|deprecates)\b',
            'Disable': r'\b(disabled|disabling|disables)\b',
            'Enable': r'\b(enabled|enabling|enables)\b',
            'Fix': r'\b(fixed|fixing|fixes)\b',
            'Improve': r'\b(improved|improving|improves)\b',
            'Make': r'\b(made|making|makes)\b',
            'Move': r'\b(moved|moving|moves)\b',
            'Rename': r'\b(renamed|renaming|renames)\b',
            'Replace': r'\b(replaced|replacing|replaces)\b',
            'Remove': r'\b(removed|removing|removes)\b',
            'Support': r'\b(supported|supporting|supports)\b',
            'Update': r'\b(updated|updating|updates)\b',
            'Upgrade': r'\b(upgraded|upgrading|upgrades)\b',
        }


class CustomFormatter(logging.Formatter):
    # logging with color
    FORMATS = {
        logging.DEBUG: grey + format + reset,
        logging.INFO: grey + format + reset,
        logging.WARNING: yellow + format + reset,
        logging.ERROR: red + format + reset,
        logging.CRITICAL: bold_red + format + reset
    }

    def format(self, record):
        log_fmt = self.FORMATS.get(record.levelno)
        formatter = logging.Formatter(log_fmt)
        return formatter.format(record)


logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)
ch = logging.StreamHandler()
ch.setLevel(logging.DEBUG)
ch.setFormatter(CustomFormatter())
logger.addHandler(ch)


def main():
    logger.info("Start check pull request ...\n")
    title = TITLE
    body = BODY.split('\r\n')
    sys.exit(1) if check_pull_request(title, body) else sys.exit(0)


def get_list(line):
    # exclude list
    # ['app']
    # ['-c', 'a b', 'a b']
    # ["-c"]
    # [1]
    # ['-c', 'System.Object[] System.Object[]', 'System.Object[] System.Object[]']
    ref = re.findall(r'([\[].*[\]])', line)
    try:
        if isinstance(ast.literal_eval(ref[0]), list):
            return True
    except:
        return False


def check_pull_request(title, body):
    if title.startswith('[') or title.startswith('{'):
        error_flag = False
        # only check title which start with `[`
        if title.startswith('['):
            error_flag = check_line(title)
        if '**History Notes**' in body:
            is_edit_history_note = False
            history_note_error_flag = False
            for line in body:
                if line.startswith('['):
                    # get component name in []
                    ref = re.findall(r'[\[](.*?)[\]]', line)
                    if ref and ref[0] not in ['Component Name 1', 'Component Name 2'] \
                            and not get_list(line) \
                            and not re.findall(r'[\[](.*?)[\]]\((.*)\)', line):  # exclude [Markdown](url)
                        is_edit_history_note = True
                        history_note_error_flag = check_line(line) or history_note_error_flag
            # If the `History Notes` is edited:
            # Use the history notes check result (history_note_error_flag), ignore the title check result (error_flag).
            error_flag = error_flag if not is_edit_history_note else history_note_error_flag
    else:
        logger.error('Pull Request title should start with [ or { , Please follow https://aka.ms/submitAzPR')
        error_flag = True
    return error_flag


def check_line(line):
    # check every line
    error_flag = False
    # Check Fix #number in title, just give a warning here, because it is not necessarily.
    if 'Fix' in line:
        sub_pattern = r'#\d'
        ref = re.findall(sub_pattern, line)
        if not ref:
            logger.warning('If it\'s related to fixing an issue, put Fix #number in title\n')
    for idx, i in enumerate(line):
        # ] } : must be followed by a space
        if i in [']', '}', ':']:
            try:
                assert line[idx + 1] == ' '
            except:
                logger.info('%s%s: missing space after %s', line, yellow, i)
                logger.error(' ' * idx + '↑')
                error_flag = True
        # az xxx commands must be enclosed in `, e.g., `az vm`
        if idx + 2 < len(line) and i == 'a' and line[idx + 1] == 'z' and line[idx + 2] == ' ':
            command = 'az '
            index = idx + 3
            while index < len(line) and line[index] != ':':
                command += line[index]
                index += 1
            try:
                assert line[idx - 1] == '`'
            except:
                logger.info('%s%s: missing ` around %s', line, yellow, command)
                logger.error(' ' * idx + '↑' + ' ' * (index - idx - 2) + '↑')
                error_flag = True
        if i == ':':
            # check extra space character before colon
            if idx - 1 >= 0 and line[idx - 1] == ' ':
                logger.info('%s%s: please delete extra space character before :', line, yellow)
                logger.error(' ' * (idx - 1) + '↑')
                error_flag = True
            # First word after the colon must be capitalized
            index = 0
            if line[idx + 1] == ' ' and line[idx + 2].islower() and line[idx + 2: idx + 5] != 'az ':
                index = idx + 2
            elif line[idx + 1] != ' ' and line[idx + 1].islower() and line[idx + 1: idx + 4] != 'az ':
                index = idx + 1
            if index:
                logger.info('%s%s: should use capital letters after :', line, yellow)
                logger.error(' ' * index + '↑')
                error_flag = True
        # --xxx parameters must be enclosed in `, e.g., `--size`
        # Add check to ensure parameters use --x-x-x format
        if i == '-' and line[idx + 1] == '-':
            param = '--'
            index = idx + 2
            while index < len(line) and line[index] != ' ':
                if line[index] == '_':
                    logger.info('%s%s: parameter should use --x-x-x format, please replace _ with -', line, yellow)
                    logger.error(' ' * index + '↑')
                    error_flag = True
                param += line[index]
                index += 1
            try:
                assert line[idx - 1] == '`'
            except:
                logger.info('%s%s: missing ` around %s', line, yellow, param)
                logger.error(' ' * idx + '↑' + ' ' * (index - idx - 2) + '↑')
                error_flag = True
        # verb check: only check the first word after ] or :
        if i in [']', ':']:
            word = ''
            c = index = idx + 1 if line[idx + 1] != ' ' else idx + 2
            while index < len(line) and line[index] != ' ':
                word += line[index]
                index += 1
            for k, v in words_to_check.items():
                if re.findall(v, word, re.IGNORECASE):
                    logger.info(line)
                    logger.error(' ' * c + '↑')
                    logger.warning(
                        'Please use the right verb of%s %s %swith %s(%s)%s simple present tense in base form '
                        'and capitalized first letter to describe what is done, '
                        'follow https://aka.ms/submitAzPR\n', red, word, yellow, green, k, yellow)
                    error_flag = True
                    break
        # check extra consecutive spaces
        if i == ' ' and (idx + 1) < len(line) and line[idx + 1] == ' ':
            logger.info('%s%s: please delete the extra space character', line, yellow)
            logger.error(' ' * (idx + 1) + '↑')
            error_flag = True

    # last character check
    if line[-1] in ['.', ',', ' ']:
        logger.info('%s%s: please delete the last character', line, yellow)
        logger.error(' ' * idx + '↑')
        error_flag = True

    return error_flag


if __name__ == '__main__':
    main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "network route-filter wait",
)
class Wait(AAZWaitCommand):
    """Place the CLI in a waiting state until a condition is met.
    """

    _aaz_info = {
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.network/routefilters/{}", "2021-08-01"],
        ]
    }

    def _handler(self, command_args):
        super()._handler(command_args)
        self._execute_operations()
        return self._output()

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )
        _args_schema.name = AAZStrArg(
            options=["-n", "--name"],
            help="Name of the route filter.",
            required=True,
            id_part="name",
        )
        _args_schema.expand = AAZStrArg(
            options=["--expand"],
            help="Expands referenced express route bgp peering resources.",
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        self.RouteFiltersGet(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=False)
        return result

    class RouteFiltersGet(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/routeFilters/{routeFilterName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "routeFilterName", self.ctx.args.name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "$expand", self.ctx.args.expand,
                ),
                **self.serialize_query_param(
                    "api-version", "2021-08-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()

            _schema_on_200 = cls._schema_on_200
            _schema_on_200.etag = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.id = AAZStrType()
            _schema_on_200.location = AAZStrType(
                flags={"required": True},
            )
            _schema_on_200.name = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200.properties = AAZObjectType(
                flags={"client_flatten": True},
            )
            _schema_on_200.tags = AAZDictType()
            _schema_on_200.type = AAZStrType(
                flags={"read_only": True},
            )

            properties = cls._schema_on_200.properties
            properties.ipv6_peerings = AAZListType(
                serialized_name="ipv6Peerings",
                flags={"read_only": True},
            )
            properties.peerings = AAZListType(
                flags={"read_only": True},
            )
            properties.provisioning_state = AAZStrType(
                serialized_name="provisioningState",
                flags={"read_only": True},
            )
            properties.rules = AAZListType()

            ipv6_peerings = cls._schema_on_200.properties.ipv6_peerings
            ipv6_peerings.Element = AAZObjectType()
            _WaitHelper._build_schema_express_route_circuit_peering_read(ipv6_peerings.Element)

            peerings = cls._schema_on_200.properties.peerings
            peerings.Element = AAZObjectType()
            _WaitHelper._build_schema_express_route_circuit_peering_read(peerings.Element)

            rules = cls._schema_on_200.properties.rules
            rules.Element = AAZObjectType()

            _element = cls._schema_on_200.properties.rules.Element
            _element.etag = AAZStrType(
                flags={"read_only": True},
            )
            _element.id = AAZStrType()
            _element.location = AAZStrType()
            _element.name = AAZStrType()
            _element.properties = AAZObjectType(
                flags={"client_flatten": True},
            )

            properties = cls._schema_on_200.properties.rules.Element.properties
            properties.access = AAZStrType(
                flags={"required": True},
            )
            properties.communities = AAZListType(
                flags={"required": True},
            )
            properties.provisioning_state = AAZStrType(
                serialized_name="provisioningState",
                flags={"read_only": True},
            )
            properties.route_filter_rule_type = AAZStrType(
                serialized_name="routeFilterRuleType",
                flags={"required": True},
            )

            communities = cls._schema_on_200.properties.rules.Element.properties.communities
            communities.Element = AAZStrType()

            tags = cls._schema_on_200.tags
            tags.Element = AAZStrType()

            return cls._schema_on_200


class _WaitHelper:
    """Helper class for Wait"""

    _schema_express_route_circuit_peering_config_read = None

    @classmethod
    def _build_schema_express_route_circuit_peering_config_read(cls, _schema):
        if cls._schema_express_route_circuit_peering_config_read is not None:
            _schema.advertised_communities = cls._schema_express_route_circuit_peering_config_read.advertised_communities
            _schema.advertised_public_prefixes = cls._schema_express_route_circuit_peering_config_read.advertised_public_prefixes
            _schema.advertised_public_prefixes_state = cls._schema_express_route_circuit_peering_config_read.advertised_public_prefixes_state
            _schema.customer_asn = cls._schema_express_route_circuit_peering_config_read.customer_asn
            _schema.legacy_mode = cls._schema_express_route_circuit_peering_config_read.legacy_mode
            _schema.routing_registry_name = cls._schema_express_route_circuit_peering_config_read.routing_registry_name
            return

        cls._schema_express_route_circuit_peering_config_read = _schema_express_route_circuit_peering_config_read = AAZObjectType()

        express_route_circuit_peering_config_read = _schema_express_route_circuit_peering_config_read
        express_route_circuit_peering_config_read.advertised_communities = AAZListType(
            serialized_name="advertisedCommunities",
        )
        express_route_circuit_peering_config_read.advertised_public_prefixes = AAZListType(
            serialized_name="advertisedPublicPrefixes",
        )
        express_route_circuit_peering_config_read.advertised_public_prefixes_state = AAZStrType(
            serialized_name="advertisedPublicPrefixesState",
            flags={"read_only": True},
        )
        express_route_circuit_peering_config_read.customer_asn = AAZIntType(
            serialized_name="customerASN",
        )
        express_route_circuit_peering_config_read.legacy_mode = AAZIntType(
            serialized_name="legacyMode",
        )
        express_route_circuit_peering_config_read.routing_registry_name = AAZStrType(
            serialized_name="routingRegistryName",
        )

        advertised_communities = _schema_express_route_circuit_peering_config_read.advertised_communities
        advertised_communities.Element = AAZStrType()

        advertised_public_prefixes = _schema_express_route_circuit_peering_config_read.advertised_public_prefixes
        advertised_public_prefixes.Element = AAZStrType()

        _schema.advertised_communities = cls._schema_express_route_circuit_peering_config_read.advertised_communities
        _schema.advertised_public_prefixes = cls._schema_express_route_circuit_peering_config_read.advertised_public_prefixes
        _schema.advertised_public_prefixes_state = cls._schema_express_route_circuit_peering_config_read.advertised_public_prefixes_state
        _schema.customer_asn = cls._schema_express_route_circuit_peering_config_read.customer_asn
        _schema.legacy_mode = cls._schema_express_route_circuit_peering_config_read.legacy_mode
        _schema.routing_registry_name = cls._schema_express_route_circuit_peering_config_read.routing_registry_name

    _schema_express_route_circuit_peering_read = None

    @classmethod
    def _build_schema_express_route_circuit_peering_read(cls, _schema):
        if cls._schema_express_route_circuit_peering_read is not None:
            _schema.etag = cls._schema_express_route_circuit_peering_read.etag
            _schema.id = cls._schema_express_route_circuit_peering_read.id
            _schema.name = cls._schema_express_route_circuit_peering_read.name
            _schema.properties = cls._schema_express_route_circuit_peering_read.properties
            _schema.type = cls._schema_express_route_circuit_peering_read.type
            return

        cls._schema_express_route_circuit_peering_read = _schema_express_route_circuit_peering_read = AAZObjectType()

        express_route_circuit_peering_read = _schema_express_route_circuit_peering_read
        express_route_circuit_peering_read.etag = AAZStrType(
            flags={"read_only": True},
        )
        express_route_circuit_peering_read.id = AAZStrType()
        express_route_circuit_peering_read.name = AAZStrType()
        express_route_circuit_peering_read.properties = AAZObjectType(
            flags={"client_flatten": True},
        )
        express_route_circuit_peering_read.type = AAZStrType(
            flags={"read_only": True},
        )

        properties = _schema_express_route_circuit_peering_read.properties
        properties.azure_asn = AAZIntType(
            serialized_name="azureASN",
        )
        properties.connections = AAZListType()
        properties.express_route_connection = AAZObjectType(
            serialized_name="expressRouteConnection",
        )
        properties.gateway_manager_etag = AAZStrType(
            serialized_name="gatewayManagerEtag",
        )
        properties.ipv6_peering_config = AAZObjectType(
            serialized_name="ipv6PeeringConfig",
        )
        properties.last_modified_by = AAZStrType(
            serialized_name="lastModifiedBy",
            flags={"read_only": True},
        )
        properties.microsoft_peering_config = AAZObjectType(
            serialized_name="microsoftPeeringConfig",
        )
        cls._build_schema_express_route_circuit_peering_config_read(properties.microsoft_peering_config)
        properties.peer_asn = AAZIntType(
            serialized_name="peerASN",
        )
        properties.peered_connections = AAZListType(
            serialized_name="peeredConnections",
            flags={"read_only": True},
        )
        properties.peering_type = AAZStrType(
            serialized_name="peeringType",
        )
        properties.primary_azure_port = AAZStrType(
            serialized_name="primaryAzurePort",
        )
        properties.primary_peer_address_prefix = AAZStrType(
            serialized_name="primaryPeerAddressPrefix",
        )
        properties.provisioning_state = AAZStrType(
            serialized_name="provisioningState",
            flags={"read_only": True},
        )
        properties.route_filter = AAZObjectType(
            serialized_name="routeFilter",
        )
        cls._build_schema_sub_resource_read(properties.route_filter)
        properties.secondary_azure_port = AAZStrType(
            serialized_name="secondaryAzurePort",
        )
        properties.secondary_peer_address_prefix = AAZStrType(
            serialized_name="secondaryPeerAddressPrefix",
        )
        properties.shared_key = AAZStrType(
            serialized_name="sharedKey",
        )
        properties.state = AAZStrType()
        properties.stats = AAZObjectType()
        properties.vlan_id = AAZIntType(
            serialized_name="vlanId",
        )

        connections = _schema_express_route_circuit_peering_read.properties.connections
        connections.Element = AAZObjectType()

        _element = _schema_express_route_circuit_peering_read.properties.connections.Element
        _element.etag = AAZStrType(
            flags={"read_only": True},
        )
        _element.id = AAZStrType()
        _element.name = AAZStrType()
        _element.properties = AAZObjectType(
            flags={"client_flatten": True},
        )
        _element.type = AAZStrType(
            flags={"read_only": True},
        )

        properties = _schema_express_route_circuit_peering_read.properties.connections.Element.properties
        properties.address_prefix = AAZStrType(
            serialized_name="addressPrefix",
        )
        properties.authorization_key = AAZStrType(
            serialized_name="authorizationKey",
        )
        properties.circuit_connection_status = AAZStrType(
            serialized_name="circuitConnectionStatus",
            flags={"read_only": True},
        )
        properties.express_route_circuit_peering = AAZObjectType(
            serialized_name="expressRouteCircuitPeering",
        )
        cls._build_schema_sub_resource_read(properties.express_route_circuit_peering)
        properties.ipv6_circuit_connection_config = AAZObjectType(
            serialized_name="ipv6CircuitConnectionConfig",
        )
        properties.peer_express_route_circuit_peering = AAZObjectType(
            serialized_name="peerExpressRouteCircuitPeering",
        )
        cls._build_schema_sub_resource_read(properties.peer_express_route_circuit_peering)
        properties.provisioning_state = AAZStrType(
            serialized_name="provisioningState",
            flags={"read_only": True},
        )

        ipv6_circuit_connection_config = _schema_express_route_circuit_peering_read.properties.connections.Element.properties.ipv6_circuit_connection_config
        ipv6_circuit_connection_config.address_prefix = AAZStrType(
            serialized_name="addressPrefix",
        )
        ipv6_circuit_connection_config.circuit_connection_status = AAZStrType(
            serialized_name="circuitConnectionStatus",
            flags={"read_only": True},
        )

        express_route_connection = _schema_express_route_circuit_peering_read.properties.express_route_connection
        express_route_connection.id = AAZStrType(
            flags={"read_only": True},
        )

        ipv6_peering_config = _schema_express_route_circuit_peering_read.properties.ipv6_peering_config
        ipv6_peering_config.microsoft_peering_config = AAZObjectType(
            serialized_name="microsoftPeeringConfig",
        )
        cls._build_schema_express_route_circuit_peering_config_read(ipv6_peering_config.microsoft_peering_config)
        ipv6_peering_config.primary_peer_address_prefix = AAZStrType(
            serialized_name="primaryPeerAddressPrefix",
        )
        ipv6_peering_config.route_filter = AAZObjectType(
            serialized_name="routeFilter",
        )
        cls._build_schema_sub_resource_read(ipv6_peering_config.route_filter)
        ipv6_peering_config.secondary_peer_address_prefix = AAZStrType(
            serialized_name="secondaryPeerAddressPrefix",
        )
        ipv6_peering_config.state = AAZStrType()

        peered_connections = _schema_express_route_circuit_peering_read.properties.peered_connections
        peered_connections.Element = AAZObjectType()

        _element = _schema_express_route_circuit_peering_read.properties.peered_connections.Element
        _element.etag = AAZStrType(
            flags={"read_only": True},
        )
        _element.id = AAZStrType()
        _element.name = AAZStrType()
        _element.properties = AAZObjectType(
            flags={"client_flatten": True},
        )
        _element.type = AAZStrType(
            flags={"read_only": True},
        )

        properties = _schema_express_route_circuit_peering_read.properties.peered_connections.Element.properties
        properties.address_prefix = AAZStrType(
            serialized_name="addressPrefix",
        )
        properties.auth_resource_guid = AAZStrType(
            serialized_name="authResourceGuid",
        )
        properties.circuit_connection_status = AAZStrType(
            serialized_name="circuitConnectionStatus",
            flags={"read_only": True},
        )
        properties.connection_name = AAZStrType(
            serialized_name="connectionName",
        )
        properties.express_route_circuit_peering = AAZObjectType(
            serialized_name="expressRouteCircuitPeering",
        )
        cls._build_schema_sub_resource_read(properties.express_route_circuit_peering)
        properties.peer_express_route_circuit_peering = AAZObjectType(
            serialized_name="peerExpressRouteCircuitPeering",
        )
        cls._build_schema_sub_resource_read(properties.peer_express_route_circuit_peering)
        properties.provisioning_state = AAZStrType(
            serialized_name="provisioningState",
            flags={"read_only": True},
        )

        stats = _schema_express_route_circuit_peering_read.properties.stats
        stats.primarybytes_in = AAZIntType(
            serialized_name="primarybytesIn",
        )
        stats.primarybytes_out = AAZIntType(
            serialized_name="primarybytesOut",
        )
        stats.secondarybytes_in = AAZIntType(
            serialized_name="secondarybytesIn",
        )
        stats.secondarybytes_out = AAZIntType(
            serialized_name="secondarybytesOut",
        )

        _schema.etag = cls._schema_express_route_circuit_peering_read.etag
        _schema.id = cls._schema_express_route_circuit_peering_read.id
        _schema.name = cls._schema_express_route_circuit_peering_read.name
        _schema.properties = cls._schema_express_route_circuit_peering_read.properties
        _schema.type = cls._schema_express_route_circuit_peering_read.type

    _schema_sub_resource_read = None

    @classmethod
    def _build_schema_sub_resource_read(cls, _schema):
        if cls._schema_sub_resource_read is not None:
            _schema.id = cls._schema_sub_resource_read.id
            return

        cls._schema_sub_resource_read = _schema_sub_resource_read = AAZObjectType()

        sub_resource_read = _schema_sub_resource_read
        sub_resource_read.id = AAZStrType()

        _schema.id = cls._schema_sub_resource_read.id


__all__ = ["Wait"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------


def mgmt_batch_account_client_factory(cli_ctx, _):
    return batch_mgmt_client_factory(cli_ctx).batch_account


def mgmt_pool_client_factory(cli_ctx, _):
    return batch_mgmt_client_factory(cli_ctx).pool


def mgmt_private_link_resource_client_factory(cli_ctx, _):
    return batch_mgmt_client_factory(cli_ctx).private_link_resource


def mgmt_private_endpoint_connection_client_factory(cli_ctx, _):
    return batch_mgmt_client_factory(cli_ctx).private_endpoint_connection


def mgmt_application_client_factory(cli_ctx, _):
    return batch_mgmt_client_factory(cli_ctx).application


def mgmt_application_package_client_factory(cli_ctx, _):
    return batch_mgmt_client_factory(cli_ctx).application_package


def mgmt_location_client_factory(cli_ctx, _):
    return batch_mgmt_client_factory(cli_ctx).location


def batch_mgmt_client_factory(cli_ctx, **_):
    from azure.mgmt.batch import BatchManagementClient
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    return get_mgmt_service_client(cli_ctx, BatchManagementClient)


def batch_data_client_factory(cli_ctx, kwargs):
    from azure.batch import BatchClient

    account_name = kwargs.pop('account_name', None)
    account_key = kwargs.pop('account_key', None)
    account_endpoint = kwargs.pop('account_endpoint', None)
    token_credential = kwargs.pop('token_credential', None)

    credential = None

    if not token_credential and not account_key:
        from azure.cli.core._profile import Profile
        profile = Profile(cli_ctx=cli_ctx)
        token_credential, _, _ = profile.get_login_credentials()

    if account_key:
        from azure.core.credentials import AzureNamedKeyCredential
        credential = AzureNamedKeyCredential(name=account_name, key=account_key)
    else:
        credential = token_credential

    if not (account_endpoint.startswith('https://') or
            account_endpoint.startswith('http://')):
        account_endpoint = 'https://' + account_endpoint

    from azure.cli.core.auth.util import resource_to_scopes
    return BatchClient(credential=credential, endpoint=account_endpoint.rstrip('/'),
                       credential_scopes=resource_to_scopes(cli_ctx.cloud.endpoints.batch_resource_id))



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=too-many-statements


from azure.cli.core.commands import CliCommandType
from azure.cli.command_modules.apim._format import (service_output_format)
from azure.cli.command_modules.apim._client_factory import (cf_service, cf_api, cf_product, cf_nv, cf_apiops,
                                                            cf_apirelease, cf_apirevision, cf_apiversionset,
                                                            cf_apischema, cf_ds, cf_graphqlapiresolver,
                                                            cf_graphqlapiresolverpolicy, cf_backend)


def load_command_table(self, _):
    service_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.apimanagement.operations#ApiManagementServiceOperations.{}',
        client_factory=cf_service
    )

    api_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.apimanagement.operations#ApiOperations.{}',
        client_factory=cf_api
    )

    api_schema = CliCommandType(
        operations_tmpl='azure.mgmt.apimanagement.operations#ApiSchemaOperations.{}',
        client_factory=cf_apischema
    )

    product_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.apimanagement.operations#ProductOperations.{}',
        client_factory=cf_product
    )

    nv_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.apimanagement.operations#NamedValueOperations.{}',
        client_factory=cf_nv
    )

    apiops_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.apimanagement.operations#ApiOperationOperations.{}',
        client_factory=cf_apiops
    )

    apirel_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.apimanagement.operations#ApiReleaseOperations.{}',
        client_factory=cf_apirelease
    )

    apirev_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.apimanagement.operations#ApiRevisionOperations.{}',
        client_factory=cf_apirevision
    )

    apivs_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.apimanagement.operations#ApiVersionSetOperations.{}',
        client_factory=cf_apiversionset
    )

    apids_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.apimanagement.operations#DeletedServicesOperations.{}',
        client_factory=cf_ds
    )

    graphql_api_resolver = CliCommandType(
        operations_tmpl='azure.mgmt.apimanagement.operations#GraphQLApiResolverOperations.{}',
        client_factory=cf_graphqlapiresolver
    )

    graphql_api_resolver_policy = CliCommandType(
        operations_tmpl='azure.mgmt.apimanagement.operations#GraphQLApiResolverPolicyOperations.{}',
        client_factory=cf_graphqlapiresolverpolicy
    )

    backend_sdk = CliCommandType(
        operations_tmpl='azure.mgmt.apimanagement.operations#BackendOperations.{}',
        client_factory=cf_backend
    )

    # pylint: disable=line-too-long
    with self.command_group('apim', service_sdk) as g:
        g.custom_command('create', 'apim_create', supports_no_wait=True,
                         table_transformer=service_output_format)
        g.custom_show_command('show', 'apim_get',
                              table_transformer=service_output_format)
        g.custom_command('list', 'apim_list',
                         table_transformer=service_output_format)
        g.command('delete', 'begin_delete',
                  confirmation=True, supports_no_wait=True)
        g.generic_update_command('update', custom_func_name='apim_update', getter_name='get',
                                 setter_name='begin_create_or_update', supports_no_wait=True)
        g.custom_command('check-name', 'apim_check_name_availability')
        g.custom_command('backup', 'apim_backup', supports_no_wait=True)
        g.custom_command('restore', 'apim_restore', supports_no_wait=True)
        g.custom_command('apply-network-updates',
                         'apim_apply_network_configuration_updates', supports_no_wait=True)
        g.wait_command('wait')

    with self.command_group('apim api', api_sdk) as g:
        g.custom_command('import', 'apim_api_import', supports_no_wait=True)
        g.custom_command('export', 'apim_api_export')
        g.custom_command('create', 'apim_api_create', supports_no_wait=True)
        g.custom_show_command('show', 'apim_api_get')
        g.custom_command('list', 'apim_api_list')
        g.custom_command('delete', 'apim_api_delete',
                         confirmation=True, supports_no_wait=True)
        g.generic_update_command('update', custom_func_name='apim_api_update',
                                 setter_name='begin_create_or_update', getter_name='get', supports_no_wait=True)
        g.wait_command('wait')

    with self.command_group('apim product api', api_sdk) as g:
        g.custom_command('list', 'apim_product_api_list')
        g.custom_command('check', 'apim_product_api_check_association')
        g.custom_command('add', 'apim_product_api_add')
        g.custom_command('delete', 'apim_product_api_delete')

    with self.command_group('apim api schema', api_schema) as g:
        g.custom_command('create', 'apim_api_schema_create', supports_no_wait=True)
        g.custom_command('delete', 'apim_api_schema_delete', confirmation=True, supports_no_wait=True)
        g.custom_show_command('show', 'apim_api_schema_get')
        g.custom_command('list', 'apim_api_schema_list')
        g.custom_command('get-etag', 'apim_api_schema_entity')
        g.wait_command('wait')

    with self.command_group('apim product', product_sdk) as g:
        g.custom_command('list', 'apim_product_list')
        g.custom_show_command('show', 'apim_product_show')
        g.custom_command('create', 'apim_product_create', supports_no_wait=True)
        g.generic_update_command('update', custom_func_name='apim_product_update', supports_no_wait=True)
        g.custom_command('delete', 'apim_product_delete', confirmation=True, supports_no_wait=True)
        g.wait_command('wait')

    with self.command_group('apim nv', nv_sdk) as g:
        g.custom_command('create', 'apim_nv_create', supports_no_wait=True)
        g.custom_show_command('show', 'apim_nv_get')
        g.custom_command('list', 'apim_nv_list')
        g.custom_command('delete', 'apim_nv_delete', confirmation=True)
        g.custom_command('show-secret', 'apim_nv_show_secret')
        g.generic_update_command('update', setter_name='begin_create_or_update', custom_func_name='apim_nv_update')
        g.wait_command('wait')

    with self.command_group('apim api operation', apiops_sdk) as g:
        g.custom_command('list', 'apim_api_operation_list')
        g.custom_show_command('show', 'apim_api_operation_get')
        g.custom_command('create', 'apim_api_operation_create')
        g.generic_update_command('update', custom_func_name='apim_api_operation_update')
        g.custom_command('delete', 'apim_api_operation_delete')

    with self.command_group('apim api release', apirel_sdk) as g:
        g.custom_command('list', 'apim_api_release_list')
        g.custom_show_command('show', 'apim_api_release_show')
        g.custom_command('create', 'apim_api_release_create')
        g.generic_update_command('update', custom_func_name='apim_api_release_update')
        g.custom_command('delete', 'apim_api_release_delete')

    with self.command_group('apim api revision', apirev_sdk) as g:
        g.custom_command('list', 'apim_api_revision_list')
        g.custom_command('create', 'apim_api_revision_create')

    with self.command_group('apim api versionset', apivs_sdk) as g:
        g.custom_command('list', 'apim_api_vs_list')
        g.custom_show_command('show', 'apim_api_vs_show')
        g.custom_command('create', 'apim_api_vs_create')
        g.generic_update_command('update', custom_func_name='apim_api_vs_update')
        g.custom_command('delete', 'apim_api_vs_delete')

    with self.command_group('apim deletedservice', apids_sdk) as g:
        g.custom_command('list', 'apim_ds_list')
        g.custom_show_command('show', 'apim_ds_get')
        g.custom_command('purge', 'apim_ds_purge')

    with self.command_group('apim graphql resolver', graphql_api_resolver) as g:
        g.custom_command('create', 'apim_graphql_resolver_create')
        g.custom_command('delete', 'apim_graphql_resolver_delete', confirmation=True)
        g.custom_show_command('show', 'apim_graphql_resolver_show')
        g.custom_command('list', 'apim_graphql_resolver_list')

    with self.command_group('apim graphql resolver policy', graphql_api_resolver_policy) as g:
        g.custom_command('create', 'apim_graphql_resolver_policy_create')
        g.custom_command('delete', 'apim_graphql_resolver_policy_delete', confirmation=True)
        g.custom_show_command('show', 'apim_graphql_resolver_policy_show')
        g.custom_command('list', 'apim_graphql_resolver_policy_list')

    with self.command_group('apim backend', backend_sdk) as g:
        g.custom_command('create', 'apim_backend_create')
        g.custom_show_command('show', 'apim_backend_show')
        g.custom_command('list', 'apim_backend_list')
        g.custom_command('delete', 'apim_backend_delete', confirmation=True)
        g.generic_update_command('update', custom_func_name='apim_backend_update')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core.util import get_default_admin_username
from azure.cli.core.aaz import has_value, register_command
from .aaz.latest.lab import Get as _LabGet, Delete as _LabDelete, CreateEnvironment as _LabVmCreate
from .aaz.latest.lab.vm import (List as _LabVmList, Show as _LabVmShow, Delete as _LabVmDelete, Start as _LabVmStart,
                                Stop as _LabVmStop, ApplyArtifacts as _LabVmApplyArtifacts, Claim as LabVmClaim,
                                Hibernate as _LabVmHibernate)
from .aaz.latest.lab.custom_image import (Show as _CustomImageShow, Delete as _CustomImageDelete,
                                          Create as _CustomImageCreate)
from .aaz.latest.lab.artifact_source import Show as _ArtifactSourceShow
from .aaz.latest.lab.vnet import Get as _LabVnetGet
from .aaz.latest.lab.formula import Show as _FormulaShow, Delete as _FormulaDelete
from .aaz.latest.lab.secret import Set as _SecretSet, List as _SecretList, Show as _SecretShow, Delete as _SecretDelete
from .aaz.latest.lab.environment import (Create as _EnvironmentCreate, List as _EnvironmentList,
                                         Show as _EnvironmentShow, Delete as _EnvironmentDelete)
from .aaz.latest.lab.arm_template import Show as _ArmTemplateShow

# pylint: disable=too-many-locals, unused-argument, too-many-statements, protected-access


@register_command(
    "lab vm create",
    is_preview=True,
)
class LabVmCreate(_LabVmCreate):
    """Create a VM in a lab.
    """
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        from azure.cli.core.aaz import AAZStrArg, AAZBoolArg, AAZFileArg
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.artifacts = AAZFileArg(
            options=["--artifacts"],
            help="Path to the JSON encoded array of artifacts to be applied. JSON encoded list of parameters."
        )
        args_schema.disk_type = AAZBoolArg(
            options=["--disk-type"],
            help="Storage type to use for virtual machine.",
            enum={'Premium': 'Premium', 'Standard': 'Standard', 'StandardSSD': 'StandardSSD'}
        )
        args_schema.os_type = AAZStrArg(
            options=["--os-type"],
            help="Type of the OS on which the custom image is based.",
            enum={"Windows": "Windows", "Linux": "Linux"},
        )
        args_schema.formula = AAZStrArg(
            options=["--formula"],
            help="Name of the formula. Use `az lab formula list` for available formulas. "
                 "Use `az lab formula` with the `--export-artifacts` flag to export and update artifacts, "
                 "then pass the results via the `--artifacts` argument."
        )
        args_schema.generate_ssh_keys = AAZBoolArg(
            options=["--generate-ssh-keys"],
            help="Generate SSH public and private key files if missing."
        )
        args_schema.image = AAZStrArg(
            options=["--image"],
            help="The name of the operating system image (gallery image name or custom image name/ID)."
                 " Use `az lab gallery-image list` for available gallery images or "
                 "`az lab custom-image list` for available custom images."
        )
        args_schema.image_type = AAZStrArg(
            options=["--image-type"],
            help="Type of the image.",
            enum={'gallery': 'gallery', 'custom': 'custom'}
        )
        args_schema.admin_username = AAZStrArg(
            options=["--admin-username"],
            help="Username for the VM admin.",
            arg_group="Authentication",
            default=get_default_admin_username()
        )
        args_schema.admin_password = AAZStrArg(
            options=["--admin-password"],
            help="Password for the VM admin.",
            arg_group="Authentication"
        )
        args_schema.authentication_type = AAZStrArg(
            options=["--authentication-type"],
            help="Type of authentication allowed for the VM.",
            enum={'password': 'password', 'ssh': 'ssh'},
            arg_group="Authentication",
            default="password"
        )
        args_schema.saved_secret = AAZStrArg(
            options=["--saved-secret"],
            help="Name of the saved secret to be used for authentication. When this value is provided, "
                 "it is used in the place of other authentication methods.",
            arg_group="Authentication"
        )
        args_schema.ip_configuration = AAZStrArg(
            options=["--ip-configuration"],
            help="Type of IP configuration to use for the VM.",
            enum={'shared': 'shared', 'public': 'public', 'private': 'private'},
            arg_group="Network"
        )
        args_schema.subnet = AAZStrArg(
            options=["--subnet"],
            help="Name of the subnet to add the VM to.",
            arg_group="Network"
        )
        args_schema.vnet_name = AAZStrArg(
            options=["--vnet-name"],
            help="Name of the virtual network to add the VM to.",
            arg_group="Network"
        )
        args_schema.name._required = True
        args_schema.location._registered = False
        args_schema.artifacts_org._registered = False
        args_schema.ssh_key._arg_group = 'Authentication'
        args_schema.os_type._registered = False
        args_schema.environment_id._registered = False
        args_schema.bulk_creation_parameters ._registered = False
        args_schema.created_date._registered = False
        args_schema.custom_image_id ._registered = False
        args_schema.data_disk_parameters._registered = False
        args_schema.gallery_image_reference._registered = False
        args_schema.disallow_public_ip_address._registered = False
        args_schema.is_authentication_with_ssh_key._registered = False
        args_schema.user_name._registered = False
        args_schema.password._registered = False
        args_schema.lab_subnet_name._registered = False
        args_schema.lab_virtual_network_id._registered = False
        args_schema.network_interface._registered = False
        args_schema.owner_object_id._registered = False
        args_schema.owner_user_principal_name._registered = False
        args_schema.plan_id._registered = False
        args_schema.schedule_parameters._registered = False
        args_schema.storage_type._registered = False
        return args_schema

    def pre_operations(self):
        import json
        from .validators import validate_lab_vm_create
        args = self.ctx.args

        if has_value(args.artifacts):
            args.artifacts_org = json.loads(args.artifacts.to_serialized_data())

        if has_value(args.disk_type):
            args.storage_type = args.disk_type

        if has_value(args.admin_password):
            args.password = args.admin_password

        if has_value(args.subnet):
            args.lab_subnet_name = args.subnet

        args.user_name = args.admin_username
        validate_lab_vm_create(self, args)

        is_ssh_authentication = args.authentication_type.to_serialized_data() == 'ssh'
        args.is_authentication_with_ssh_key = is_ssh_authentication


class LabVmList(_LabVmList):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        from azure.cli.core.aaz import AAZStrArg, AAZBoolArg
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.claimable = AAZBoolArg(
            options=["--claimable"],
            help="List only claimable virtual machines in the lab. Cannot be used with `--filters`."
        )
        args_schema.all = AAZBoolArg(
            options=["--all"],
            help="List all virtual machines in the lab. Cannot be used with `--filters`."
        )
        args_schema.environment = AAZStrArg(
            options=["--environment"],
            help="Name or ID of the environment to list virtual machines in. Cannot be used with `--filters`.",
            arg_group="Filter"
        )
        args_schema.object_id = AAZStrArg(
            options=["--object-id"],
            help="Object ID of the owner to list VMs for."
        )
        args_schema.filters._arg_group = "Filter"
        return args_schema

    def pre_operations(self):
        from .validators import validate_lab_vm_list
        args = self.ctx.args
        validate_lab_vm_list(self, args)


class LabVmShow(_LabVmShow):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.name._id_part = None
        args_schema.lab_name._id_part = None
        return args_schema


class LabVmDelete(_LabVmDelete):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.name._id_part = None
        args_schema.lab_name._id_part = None
        return args_schema


class LabVmStart(_LabVmStart):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.name._id_part = None
        args_schema.lab_name._id_part = None
        return args_schema


class LabVmStop(_LabVmStop):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.name._id_part = None
        args_schema.lab_name._id_part = None
        return args_schema


class LabVmHibernate(_LabVmHibernate):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.name._id_part = None
        args_schema.lab_name._id_part = None
        return args_schema


class LabVmApplyArtifacts(_LabVmApplyArtifacts):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        from azure.cli.core.aaz import AAZFileArg
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.artifacts = AAZFileArg(
            options=["--artifacts"],
            help="Path to the JSON encoded array of artifacts to be applied. JSON encoded list of parameters."
        )
        args_schema.name._id_part = None
        args_schema.lab_name._id_part = None
        args_schema.artifacts_org._registered = False
        return args_schema

    def pre_operations(self):
        import json
        args = self.ctx.args
        if has_value(args.artifacts):
            args.artifacts_org = json.loads(args.artifacts.to_serialized_data())


def claim_vm(cmd, lab_name=None, name=None, resource_group_name=None):
    """ Command to claim a VM in the Azure DevTest Lab"""
    if name is None:
        from .aaz.latest.lab import ClaimAnyVm
        result = ClaimAnyVm(cli_ctx=cmd.cli_ctx)(command_args={
            "name": lab_name,
            "resource_group": resource_group_name
        })
        return result
    result = LabVmClaim(cli_ctx=cmd.cli_ctx)(command_args={
        "name": name,
        "lab_name": lab_name,
        "resource_group": resource_group_name
    })
    return result


def _export_parameters(arm_template):
    parameters = []
    if arm_template and arm_template.get('contents') and arm_template['contents'].get('parameters'):
        default_values = {}
        if arm_template.get('parametersValueFilesInfo'):
            for parameter_value_file_info in arm_template.get('parametersValueFilesInfo'):
                if isinstance(parameter_value_file_info['parametersValueInfo'], dict):
                    for k in parameter_value_file_info['parametersValueInfo']:
                        default_values[k] = parameter_value_file_info['parametersValueInfo'][k].get('value', "")

        if isinstance(arm_template['contents']['parameters'], dict):
            for k in arm_template['contents']['parameters']:
                param = {}
                param['name'] = k
                param['value'] = default_values.get(k, "")
                parameters.append(param)
    return parameters


class LabGet(_LabGet):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.name._id_part = None
        return args_schema


class LabDelete(_LabDelete):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.name._id_part = None
        return args_schema


class CustomImageCreate(_CustomImageCreate):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        from azure.cli.core.aaz import AAZStrArg, AAZResourceIdArg, AAZResourceIdArgFormat
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.source_vm_id = AAZResourceIdArg(
            options=["--source-vm-id"],
            help="The resource ID of a virtual machine in the provided lab.",
            fmt=AAZResourceIdArgFormat(
                template="/subscriptions/{}/resourcegroups/{}/providers/microsoft.devtestlab/labs/{}/virtualmachines/{}"
            ),
            required=True
        )
        args_schema.os_type = AAZStrArg(
            options=["--os-type"],
            help="Type of the OS on which the custom image is based.",
            enum={"Windows": "Windows", "Linux": "Linux"},
            required=True
        )
        args_schema.os_state = AAZStrArg(
            options=["--os-state"],
            help="The current state of the virtual machine. "
                 "For Windows virtual machines: NonSysprepped, SysprepRequested, SysprepApplied. "
                 "For Linux virtual machines: NonDeprovisioned, DeprovisionRequested, DeprovisionApplied.",
            required=True
        )
        args_schema.vm._registered = False
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        args.vm.source_vm_id = args.source_vm_id
        if args.os_type.to_serialized_data().lower() == "linux":
            args.vm.linux_os_info.linux_os_state = args.os_state
        if args.os_type.to_serialized_data().lower() == "windows":
            args.vm.windows_os_info.windows_os_state = args.os_state


class CustomImageShow(_CustomImageShow):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.lab_name._id_part = None
        args_schema.name._id_part = None
        return args_schema


class CustomImageDelete(_CustomImageDelete):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.lab_name._id_part = None
        args_schema.name._id_part = None
        return args_schema


class ArtifactSourceShow(_ArtifactSourceShow):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.lab_name._id_part = None
        args_schema.name._id_part = None
        return args_schema


class LabVnetGet(_LabVnetGet):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.lab_name._id_part = None
        args_schema.name._id_part = None
        return args_schema


class FormulaShow(_FormulaShow):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.lab_name._id_part = None
        args_schema.name._id_part = None
        return args_schema


@register_command(
    "lab formula export-artifacts",
    is_preview=True,
)
class FormulaExportArtifacts(_FormulaShow):
    """Export artifacts from a formula.
    """
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.lab_name._id_part = None
        args_schema.name._id_part = None
        return args_schema


class FormulaDelete(_FormulaDelete):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.lab_name._id_part = None
        args_schema.name._id_part = None
        return args_schema


class SecretSet(_SecretSet):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.lab_name._id_part = None
        args_schema.name._id_part = None
        args_schema.user_name._required = False
        args_schema.user_name._registered = False
        args_schema.value._required = True
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        args.user_name = "@me"


class SecretList(_SecretList):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.user_name._required = False
        args_schema.user_name._registered = False
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        args.user_name = "@me"


class SecretShow(_SecretShow):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.name._id_part = None
        args_schema.lab_name._id_part = None
        args_schema.user_name._required = False
        args_schema.user_name._registered = False
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        args.user_name = "@me"


class SecretDelete(_SecretDelete):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.name._id_part = None
        args_schema.lab_name._id_part = None
        args_schema.user_name._required = False
        args_schema.user_name._registered = False
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        args.user_name = "@me"


class EnvironmentCreate(_EnvironmentCreate):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        from azure.cli.core.aaz import AAZStrArg, AAZFileArg
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.arm_template = AAZStrArg(
            options=["--arm-template"],
            help="Name or ID of the ARM template in the lab.",
            required=True
        )
        args_schema.artifact_source_name = AAZStrArg(
            options=["--artifact-source-name"],
            help="Name of the artifact source in the lab.  Values from: az lab artifact-source list."
        )
        args_schema.parameters = AAZFileArg(
            options=["--parameters"],
            help="Path to the parameters of the azure resource manager template. JSON encoded list of parameters."
        )
        args_schema.user_name._required = False
        args_schema.user_name._registered = False
        args_schema.deployment_properties._registered = False
        return args_schema

    def pre_operations(self):
        import json
        from azure.mgmt.core.tools import resource_id, is_valid_resource_id
        from azure.cli.core.azclierror import RequiredArgumentMissingError
        from azure.cli.core.commands.client_factory import get_subscription_id
        args = self.ctx.args
        args.user_name = "@me"

        if has_value(args.parameters):
            args.deployment_properties.parameters = json.loads(args.parameters.to_serialized_data())

        if not is_valid_resource_id(args.arm_template.to_serialized_data()):
            if not has_value(args.artifact_source_name):
                raise RequiredArgumentMissingError("--artifact-source-name is required "
                                                   "when name is provided for --arm-template")
            args.arm_template = resource_id(subscription=get_subscription_id(self.cli_ctx),
                                            resource_group=args.resource_group.to_serialized_data(),
                                            namespace='Microsoft.DevTestLab',
                                            type='labs',
                                            name=args.lab_name.to_serialized_data(),
                                            child_type_1='artifactSources',
                                            child_name_1=args.artifact_source_name.to_serialized_data(),
                                            child_type_2='armTemplates',
                                            child_name_2=args.arm_template.to_serialized_data())
        args.deployment_properties.arm_template_id = args.arm_template


class EnvironmentList(_EnvironmentList):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.user_name._required = False
        args_schema.user_name._registered = False
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        args.user_name = "@me"


class EnvironmentShow(_EnvironmentShow):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.name._id_part = None
        args_schema.lab_name._id_part = None
        args_schema.user_name._required = False
        args_schema.user_name._registered = False
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        args.user_name = "@me"


class EnvironmentDelete(_EnvironmentDelete):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.name._id_part = None
        args_schema.lab_name._id_part = None
        args_schema.user_name._required = False
        args_schema.user_name._registered = False
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        args.user_name = "@me"


class ArmTemplateShow(_ArmTemplateShow):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        from azure.cli.core.aaz import AAZBoolArg
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.export_parameters = AAZBoolArg(
            options=["--export-parameters"],
            help="Whether or not to export parameters template."
        )
        args_schema.name._id_part = None
        args_schema.lab_name._id_part = None
        return args_schema

    def _output(self, *args, **kwargs):
        args = self.ctx.args
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        if has_value(args.export_parameters) and args.export_parameters.to_serialized_data() is True:
            result = _export_parameters(result)
        return result
